import { Children, useMemo } from "react";
import { Divider } from "../../divider";
import { Item, Section } from "../../collection";
import { TooltipTrigger, parseTooltipTrigger } from "../../tooltip";
import { isNil, resolveChildren } from "../../shared";
export let NodeType;

(function (NodeType) {
  NodeType["item"] = "item";
  NodeType["section"] = "section";
  NodeType["divider"] = "divider";
})(NodeType || (NodeType = {}));

export function isSection(node) {
  return node.type === NodeType.section;
}
export function isDivider(node) {
  return node.type === NodeType.divider;
}
export function isItem(node) {
  return node.type === NodeType.item;
}
export function createCollectionItem({
  content,
  elementType,
  index,
  key,
  props,
  ref
}) {
  return {
    content,
    elementType,
    index,
    key,
    props,
    ref,
    type: NodeType.item
  };
}
export class CollectionBuilder {
  parseItem(element, incrementIndex) {
    const {
      children,
      ...props
    } = element.props;
    const index = incrementIndex();
    return {
      content: children,
      // Use a custom type if available otherwise let the final component choose his type.
      elementType: element.type !== Item ? element.type : undefined,
      index,
      key: !isNil(element.key) ? element.key.toString().replace(".", "").replace("$", "") : index.toString(),
      props,
      ref: element.ref,
      type: NodeType.item
    };
  }

  parseSection(element, incrementIndex) {
    const {
      children,
      ...props
    } = element.props;
    const index = incrementIndex(); // eslint-disable-next-line @typescript-eslint/no-this-alias

    const that = this;
    const items = Children.toArray(resolveChildren(children)).filter(x => x).map(x => that.parseItem(x, incrementIndex));
    return {
      // Use a custom type if available otherwise let the final component choose his type.
      elementType: element.type !== Section ? element.type : undefined,
      index,
      items,
      key: index.toString(),
      props,
      ref: element.ref,
      type: NodeType.section
    };
  }

  parseDivider(element, incrementIndex) {
    const {
      children,
      ...props
    } = element.props;
    const index = incrementIndex();
    return {
      content: children,
      elementType: Divider,
      index,
      key: index.toString(),
      props,
      ref: element.ref,
      type: NodeType.divider
    };
  }

  parseTooltip(element, incrementIndex) {
    const {
      children,
      ...props
    } = element.props;
    const [item, tooltip] = parseTooltipTrigger(children);
    const parsedItem = this.parseItem(item, incrementIndex);
    parsedItem.tooltip = {
      content: tooltip,
      props
    };
    return parsedItem;
  }

  build(children) {
    if (isNil(children)) {
      return [];
    }

    let index = 0;
    const elements = resolveChildren(children);

    const incrementIndex = () => {
      return index++;
    }; // eslint-disable-next-line @typescript-eslint/no-this-alias


    const that = this;
    return Children.toArray(elements).filter(x => x).map(element => {
      switch (element.type) {
        case Section:
          return that.parseSection(element, incrementIndex);

        case Divider:
          return that.parseDivider(element, incrementIndex);

        case TooltipTrigger:
          return that.parseTooltip(element, incrementIndex);

        default:
          return that.parseItem(element, incrementIndex);
      }
    });
  }

}
export function useCollection(children) {
  const builder = useMemo(() => new CollectionBuilder(), []);
  return useMemo(() => builder.build(children), [builder, children]);
}