import { FocusTarget, Keys, arrayify, isNil, isNumber, mergeProps, useAutoFocus, useChainedEventCallback, useControllableState, useEventCallback, useId, useMergedRefs, useRawSlots, useRefState } from "../../shared";
import { useCallback, useMemo } from "react";
import { OptionKeyProp } from "../../listbox";
import { usePopup, useTriggerWidth } from "../../overlay";
import { useCollection, useOnlyCollectionItems } from "../../collection";
export function useSelect(children, {
  align = "start",
  allowFlip,
  allowPreventOverflow,
  allowResponsiveMenuWidth = true,
  ariaDescribedBy,
  ariaLabel,
  ariaLabelledBy,
  autoFocus,
  defaultOpen,
  defaultSelectedKey,
  direction = "bottom",
  disabled,
  id,
  onOpenChange,
  onSelectionChange,
  open: openProp,
  overlayProps: {
    id: menuId,
    width: menuWidth,
    ...menuProps
  } = {},
  readOnly,
  ref,
  selectedKey: selectedKeyProp,
  validationState
}) {
  var _ref, _ref2, _text$props$children, _text$props;

  const [selectedKey, setSelectedKey] = useControllableState(selectedKeyProp, defaultSelectedKey, null);
  const [focusTargetRef, setFocusTarget] = useRefState(FocusTarget.first);
  const handleOpenChange = useChainedEventCallback(onOpenChange, (event, isVisible) => {
    // When the select is closed because of a blur or outside click event, reset the focus target.
    if (!isVisible) {
      setFocusTarget(FocusTarget.first);
    }
  });
  const {
    focusScope,
    isOpen,
    overlayProps,
    setIsOpen,
    triggerProps: {
      ref: popupTriggerRef,
      ...triggerProps
    }
  } = usePopup("listbox", {
    allowFlip,
    allowPreventOverflow,
    defaultOpen,
    disabled: disabled || readOnly,
    hideOnEscape: true,
    hideOnLeave: true,
    id: menuId,
    keyProp: OptionKeyProp,
    offset: [0, 4],
    onOpenChange: handleOpenChange,
    open: openProp,
    position: `${direction}-${align}`,
    restoreFocus: true,
    trigger: "click"
  });
  const [triggerWidthRef, triggerWidth] = useTriggerWidth({
    isDisabled: !allowResponsiveMenuWidth || !isNil(menuWidth)
  });
  const triggerRef = useMergedRefs(ref, popupTriggerRef, triggerWidthRef);
  const updateSelectedKey = useCallback((event, newKeys) => {
    var _newKeys$;

    const newKey = (_newKeys$ = newKeys[0]) !== null && _newKeys$ !== void 0 ? _newKeys$ : null;

    if (newKey !== selectedKey) {
      setSelectedKey(newKey);

      if (!isNil(onSelectionChange)) {
        onSelectionChange(event, newKey);
      }
    }
  }, [selectedKey, setSelectedKey, onSelectionChange]);
  const open = useCallback((event, focusTarget) => {
    setFocusTarget(focusTarget);
    setIsOpen(event, true);
  }, [setIsOpen, setFocusTarget]);
  const close = useCallback(event => {
    setIsOpen(event, false);
  }, [setIsOpen]); // Open the menu on up & down arrow keydown.

  const handleTriggerKeyDown = useEventCallback(event => {
    switch (event.key) {
      case Keys.arrowDown:
        event.preventDefault();
        open(event, FocusTarget.first);
        break;

      case Keys.arrowUp:
        event.preventDefault();
        open(event, FocusTarget.last);
        break;
    }
  }); // Keep the selected key in sync with the listbox.

  const handleListboxSelectionChange = useEventCallback((event, newValue) => {
    updateSelectedKey(event, newValue);
    close(event);
  });
  useAutoFocus(triggerRef, {
    delay: isNumber(autoFocus) ? autoFocus : undefined,
    isDisabled: !autoFocus || isOpen
  });
  const nodes = useCollection(children);
  const items = useOnlyCollectionItems(nodes);
  const selectedItem = useMemo(() => items.find(x => x.key === selectedKey), [items, selectedKey]);
  const {
    avatar,
    "end-icon": endIcon,
    icon,
    stringValue,
    text
  } = useRawSlots(selectedItem === null || selectedItem === void 0 ? void 0 : selectedItem.content, ["icon", "avatar", "text", "end-icon"]);
  const triggerId = useId(id, "o-ui-select-trigger");
  return {
    close,
    focusScope,
    isOpen,
    listboxProps: {
      "aria-describedby": ariaDescribedBy,
      "aria-label": ariaLabel,
      "aria-labelledby": isNil(ariaLabel) ? ariaLabelledBy !== null && ariaLabelledBy !== void 0 ? ariaLabelledBy : triggerId : undefined,
      // Must be conditional to isOpen otherwise it will steal the focus from the trigger when selecting
      // a value because the listbox re-render before the exit animation is done.
      autoFocus: isOpen,
      defaultFocusTarget: focusTargetRef.current,
      fluid: true,
      focusOnHover: true,
      nodes,
      onSelectionChange: handleListboxSelectionChange,
      selectedKeys: useMemo(() => arrayify(selectedKey), [selectedKey]),
      validationState
    },
    open,
    overlayProps: mergeProps({
      className: "o-ui-select-menu",
      width: (_ref = menuWidth !== null && menuWidth !== void 0 ? menuWidth : triggerWidth) !== null && _ref !== void 0 ? _ref : undefined
    }, menuProps, overlayProps),
    selectedItem: isNil(selectedItem) ? undefined : {
      avatar,
      endIcon,
      icon,
      text: (_ref2 = (_text$props$children = text === null || text === void 0 ? void 0 : (_text$props = text.props) === null || _text$props === void 0 ? void 0 : _text$props.children) !== null && _text$props$children !== void 0 ? _text$props$children : stringValue) !== null && _ref2 !== void 0 ? _ref2 : ""
    },
    selectedKey,
    setSelectedKey: updateSelectedKey,
    triggerProps: mergeProps({
      "aria-describedby": ariaDescribedBy,
      "aria-label": ariaLabel,
      "aria-labelledby": isNil(ariaLabel) ? ariaLabelledBy : undefined,
      disabled,
      id: triggerId,
      onKeyDown: !isOpen ? handleTriggerKeyDown : undefined,
      ref: triggerRef
    }, triggerProps)
  };
}