import { createChainedFunction } from "./useChainedEventCallback";
import { isFunction, isNil, isUndefined } from "./assertions";
import { mergeClasses } from "./mergeClasses";
import { mergeRefs } from "./useMergedRefs";
// Useful to compose a weak map key with multiple objects.
export class CompositeKeyWeakMap {
  constructor() {
    this.root = new WeakMap();
  }

  set(keys, value) {
    let node = this.root;

    for (let i = 0; i < keys.length; i += 1) {
      const key = keys[i];
      const map = node.map || node;

      if (!map.has(key)) {
        const child = {
          map: new WeakMap(),
          value: undefined
        };
        map.set(key, child);
        node = child;
      } else {
        node = map.get(key);
      }
    }

    node.value = value;
  }

  get(keys) {
    let node = this.root;

    for (let i = 0; i < keys.length; i += 1) {
      const map = node.map || node;
      node = map.get(keys[i]);

      if (isUndefined(node)) {
        return node;
      }
    }

    return node.value;
  }

  has(keys) {
    return !isUndefined(this.get(keys));
  }

} ////////

const cache = new CompositeKeyWeakMap();

function memoMerge(x, y, fct) {
  if (isNil(x) && isNil(y)) {
    return undefined;
  }

  if (isNil(x)) {
    return y;
  }

  if (isNil(y)) {
    return x;
  }

  const key = [x, y];
  const value = cache.get(key);

  if (!isUndefined(value)) {
    return value;
  }

  const mergeResult = fct(y, x);
  cache.set(key, mergeResult);
  return mergeResult;
}

function merge(props, newProps) {
  Object.keys(newProps).forEach(x => {
    if (!isUndefined(newProps[x])) {
      if (!isNil(props[x])) {
        if (x === "className") {
          props[x] = mergeClasses(props[x], newProps[x]);
        } else if (x === "style") {
          props[x] = { ...newProps[x],
            ...props[x]
          };
        } else if (x === "ref") {
          props[x] = memoMerge(props[x], newProps[x], mergeRefs);
        } else if (x.startsWith("on") && isFunction(props[x])) {
          props[x] = memoMerge(props[x], newProps[x], createChainedFunction);
        }
      } else {
        props[x] = newProps[x];
      }
    }
  });
  return props;
}
/*
Took from: https://stackoverflow.com/questions/51603250/typescript-3-parameter-list-intersection-type/51604379#51604379.

Convert an array of obj to a union of all the possibility.

Before:
[{
    a?: string;
    b?: string;
}, {
    c: string;
}, {
    d: boolean;
}]

After:
{
    a?: string;
    b?: string;
} | {
    c: string;
} | {
    d: boolean;
}
*/


export function mergeProps(...args) {
  let result = {};
  args.forEach(x => {
    result = merge(result, x);
  });
  return result;
}